<?php

use CKPL\Pay\Exception\EndpointException;
use CKPL\Pay\Exception\ClientException;
use CKPL\Pay\Exception\Definition\PaymentException;

/**
 * User controller.
 */
class ControllerExtensionPaymentConotoxiaPay extends Controller
{
    /**
     * Name of plugin.
     */
    const PLUGIN_NAME = 'conotoxiapay';

    /**
     * Order status - refund.
     */
    const ORDER_STATUS_REFUNDED = 11;

    /**
     * @var Log $logger
     */
    private $logger;

    /**
     * Prepare log file
     */
    private function initLogger()
    {
        $this->logger = new Log('conotoxiapay.log');
    }

    /**
     * Action, which is executed during checkout view.
     */
    public function index()
    {
        $this->load->model('checkout/order');
        $this->load->model('setting/setting');
        $this->load->language($this->getExtensionName());

        $data['text_button_confirm'] = $this->language->get('text_conotoxia_confirm_button');

        $data['action'] = $this->createUrl('preparing');
        if (isset($this->session->data['user_token'])) {
            $data['action'] .= '&user_token=' . $this->session->data['user_token'];
        }

        return $this->load->view('extension/payment/conotoxiapay', $data);
    }

    /**
     * Action, which is executed during preparing request for ConotoxiaPay.
     */
    public function preparing()
    {
        $this->load->model('checkout/order');
        $this->load->model('setting/setting');
        $this->load->language($this->getExtensionName());
        $this->load->library('conotoxiapayment');
        $this->initLogger();

        try {
            $order = $this->model_checkout_order->getOrder($this->session->data['order_id']);

            $conotoxiaPayment = new ConotoxiaPayment($this->registry);
            $paymentResponse = $conotoxiaPayment->paymentProcess($order);

            $this->model_checkout_order->addOrderHistory($order['order_id'],
                $this->config->get($this->getConfigKey('status_processing')));

            if (!empty($paymentResponse)) {
                $this->response->redirect($paymentResponse->getApproveUrl());
            } else {
                throw new EndpointException();
            }
        } catch (EndpointException | PaymentException | ClientException | Exception $e) {
            $this->logger->write(get_class($e) . ' ' . $e->getMessage());
            $this->session->data['error'] = 'Problem with making payments, please contact the store administratore!';
            $this->response->redirect(HTTPS_SERVER . 'index.php?route=checkout/checkout');
        }
    }

    /**
     * Action, which is executed when customer is coming back to shop from ConotoxiaPay.
     */
    public function back()
    {
        $this->load->language($this->getExtensionName());
        $this->load->model('checkout/order');
        $this->load->model('setting/setting');

        $this->redirectClient(HTTPS_SERVER . 'index.php?route=account/order', 'checkout/success');
    }

    /**
     * Action, which is executed when customer received the error and is coming back to shop from ConotoxiaPay.
     */
    public function failed()
    {
        $this->load->model('checkout/order');
        $this->load->model('setting/setting');
        $this->load->language($this->getExtensionName());
        $this->initLogger();

        if ($this->customer->isLogged()) {
            $data['back_redirect_url'] = HTTPS_SERVER . 'index.php?route=account/order';
        } else {
            $data['back_redirect_url'] = HTTPS_SERVER . 'index.php?route=common/home';
        }
        $data['back_button_text'] = $this->language->get('back_button_text');
        $data['message'] = $this->language->get('payment_failed_message');

        $data['header'] = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer'] = $this->load->controller('common/footer');

        $this->logger->write($this->language->get('payment_failed_message'));
        $this->response->setOutput($this->load->view($this->getExtensionName() . '_failed', $data));
    }

    /**
     * Action executed when ConotoxiaPay send requests with URLC status confirmation.
     */
    public function confirm()
    {
        $this->load->model('checkout/order');
        $this->load->model('setting/setting');
        $this->load->library('conotoxiapayment');
        $this->initLogger();

        $conotoxiaPayment = new ConotoxiaPayment($this->registry);

        if ($this->request->server['REQUEST_METHOD'] == 'POST') {
            $input = file_get_contents('php://input');
            $status = $conotoxiaPayment->getConfirmStatus($input);
            $paymentStatus = $conotoxiaPayment->getPaymentStatus($status);
            $order = $this->model_checkout_order->getOrder($conotoxiaPayment->getConfirmOrderId($status));

            if ($order['order_status'] !== 'Complete' && $order['order_status'] !== 'Canceled') {
                if ($paymentStatus) {
                    $this->model_checkout_order->addOrderHistory($order['order_id'], $paymentStatus);
                } else {
                    $this->logger->write($this->language->get('notification_code_error'));
                }
            }
            $this->sendResponse('HTTP/1.1 200 OK', 'OK');
        } else {
            $this->sendResponse('HTTP/1.1 400 Bad Request', 'ERROR');
        }
    }

    /**
     * Action executed when ConotoxiaPay send requests with URLC - status refunded.
     */
    public function refund()
    {
        $this->load->model('checkout/order');
        $this->load->model('setting/setting');
        $this->load->library('conotoxiapayment');
        $this->initLogger();

        $conotoxiaPayment = new ConotoxiaPayment($this->registry);

        if ($this->request->server['REQUEST_METHOD'] == 'POST') {
            $input = file_get_contents('php://input');
            $status = $conotoxiaPayment->getRefundStatus($input);
            $order = $this->model_checkout_order->getOrder($conotoxiaPayment->getRefundOrderId($status));

            $this->model_checkout_order->addOrderHistory($order['order_id'], self::ORDER_STATUS_REFUNDED);

            $this->sendResponse('HTTP/1.1 200 OK', 'OK');
        } else {
            $this->sendResponse('HTTP/1.1 400 Bad Request', 'ERROR');
        }
    }

    /**
     * Creates order in shop.
     */
    public function createOrder()
    {
        $this->load->model('checkout/order');
        $this->model_checkout_order->addOrderHistory(
            $this->order['order_id'],
            $this->config->get($this->getConfigKey('status_processing'))
        );
    }

    /**
     * Redirect the client to a good URL depending on whether the client is logged in.
     *
     * @param string $loggedUrl
     * @param string $notLoggedUrl
     */
    private function redirectClient(string $loggedUrl, string $notLoggedUrl)
    {
        if ($this->customer->isLogged()) {
            $this->response->redirect($loggedUrl);
        } else {
            $this->load->controller($notLoggedUrl, array());
        }
    }

    /**
     * Send response to ConotoxiaPay after received notification witch confirm or refund
     *
     * @param string $header
     * @param string $message
     */
    private function sendResponse(string $header, string $message)
    {
        header($header);
        echo $message;

        ob_flush();
    }

    /**
     * Returns full path of Conotoxia payment extension.
     *
     * @return string
     */
    private function getExtensionName(): string
    {
        return 'extension/payment/' . self::PLUGIN_NAME;
    }

    /**
     * Get full name of value used in configuration
     * @param string $key Name of value
     * @return string
     */
    private function getConfigKey(string $key): string
    {
        return 'payment_' . self::PLUGIN_NAME . '_' . $key;
    }

    /**
     * Create an url with given value as 'route' parameter
     * @param string $route Specific route for url
     * @return string
     */
    private function createUrl(string $route): string
    {
        return HTTPS_SERVER . 'index.php?route=' . $this->getExtensionName() . '/' . $route;
    }
}