<?php

declare(strict_types=1);

require_once dirname(__DIR__) . '/library/conotoxiapay/internals/autoload.php';

use CKPL\Pay\Pay;
use CKPL\Pay\Configuration\Factory\ConfigurationFactory;
use CKPL\Pay\Storage\FileStorage;
use CKPL\Pay\Exception\MerchantManagerException;
use CKPL\Pay\Configuration\ConfigurationInterface;
use CKPL\Pay\Model\Response\CreatedPaymentResponseModel;

/**
 * PaymentObject class
 */
class ConotoxiaPayment
{
    /**
     * Name of plugin.
     */
    const PLUGIN_NAME = 'conotoxiapay';

    /**
     * Type of algorithm.
     */
    const CONOTOXIA_ALGORITHM = 'SHA512';

    /**
     * Table containing available order statuses.
     */
    const ORDER_STATUS = [
        'NEW' => 0,
        'COMPLETED' => 5,
        'REJECTED' => 7,
        'CANCELLED' => 7,
        'UNKNOWN' => 0,
    ];

    /**
     * Registry class with shop environment data.
     *
     * @var Registry
     */
    private $registry;

    /**
     * @var Pay $pay
     */
    private $pay;

    /**
     * Prepare object for using.
     *
     * @param Registry $registry Registry object with environment data
     */
    public function __construct(Registry $registry)
    {
        $this->registry = $registry;
        $this->pay = new Pay($this->setPaymentConfig());
    }

    /**
     * Returns value of property, using Registry object.
     *
     * @param string $key Name of property
     *
     * @return type
     */
    public function __get($key)
    {
        return $this->registry->get($key);
    }

    /**
     * @return ConfigurationInterface
     */
    public function setPaymentConfig(): ConfigurationInterface
    {

        try {
            return ConfigurationFactory::fromArray([
                ConfigurationInterface::HOST => $this->getConfigKey('payment_host'),
                ConfigurationInterface::OIDC => $this->getConfigKey('payment_oidc'),
                ConfigurationInterface::CLIENT_ID => $this->getConfigKey('client_id'),
                ConfigurationInterface::CLIENT_SECRET => $this->getConfigKey('client_secret'),
                ConfigurationInterface::POINT_OF_SALE => $this->getConfigKey('point_of_sale'),
                ConfigurationInterface::PRIVATE_KEY => $this->getConfigKey('rsa_priv_key'),
                ConfigurationInterface::PUBLIC_KEY => $this->getConfigKey('rsa_pub_key'),
                ConfigurationInterface::STORAGE => new FileStorage(__DIR__ . '/storage.json'),
                ConfigurationInterface::SIGN_ALGORITHM => static::CONOTOXIA_ALGORITHM,
            ]);
        } catch (\Exception $exception) {
            echo $exception->getMessage(), PHP_EOL;
        }
    }

    /**
     * @param string $input
     * @return \CKPL\Pay\Payment\Status\StatusInterface
     */
    public function getConfirmStatus(string $input): \CKPL\Pay\Payment\Status\StatusInterface
    {
        return $this->pay->payments()->getPaymentStatus($input);
    }

    /**
     * @param string $input
     * @return \CKPL\Pay\Refund\Status\StatusInterface
     */
    public function getRefundStatus(string $input): \CKPL\Pay\Refund\Status\StatusInterface
    {
        return $this->pay->refunds()->getRefundStatus($input);
    }

    /**
     * @param \CKPL\Pay\Payment\Status\StatusInterface $status
     * @return int
     */
    public function getConfirmOrderId(\CKPL\Pay\Payment\Status\StatusInterface $status): int
    {
        return (int)$status->getExternalPaymentId();
    }

    /**
     * @param \CKPL\Pay\Refund\Status\StatusInterface $status
     * @return int
     */
    public function getRefundOrderId(\CKPL\Pay\Refund\Status\StatusInterface $status): int
    {
        return (int)$status->getExternalRefundId();
    }

    /**
     * @param \CKPL\Pay\Payment\Status\StatusInterface $status
     * @return int
     */
    public function getPaymentStatus(\CKPL\Pay\Payment\Status\StatusInterface $status): int
    {
        if ($status->isNew()) {
            $result = self::ORDER_STATUS['NEW'];
        } elseif ($status->isCompleted()) {
            $result = self::ORDER_STATUS['COMPLETED'];
        } elseif ($status->isRejected()) {
            $result = self::ORDER_STATUS['REJECTED'];
        } elseif ($status->isCancelled()) {
            $result = self::ORDER_STATUS['CANCELLED'];
        } else {
            $result = self::ORDER_STATUS['UNKNOWN'];
        }

        return $result;
    }

    /**
     * @param array $order
     * @return CreatedPaymentResponseModel
     * @throws \CKPL\Pay\Exception\ClientException
     * @throws \CKPL\Pay\Exception\Exception
     * @throws \CKPL\Pay\Exception\Definition\AmountException
     * @throws \CKPL\Pay\Exception\Definition\PaymentCustomerException
     * @throws \CKPL\Pay\Exception\Definition\PaymentException
     */
    public function paymentProcess(array $order): CreatedPaymentResponseModel
    {

        return $this->pay->payments()->makePayment(PaymentFactory::create($this->pay, $order));
    }

    /**
     * Save public key from service to local json file.
     */
    public function savePublicKeyToStorage()
    {
        $this->pay->pickSignatureKey();
    }

    /**
     * Send public key from shop to service.
     */
    public function setPublicKey()
    {
        try {
            $this->pay->merchant()->pickPublicKeyId();
        } catch (MerchantManagerException $exception) {
            $this->pay->merchant()->sendPublicKey();
        }
    }

    /**
     * Get full name of value used in configuration
     * @param string $key Name of value
     * @return string
     */
    private function getConfigKey(string $key): string
    {
        return $this->registry->get('config')->get('payment_' . self::PLUGIN_NAME . '_' . $key);
    }
}
